//
// (c) 2006 DS Data Systems UK Ltd, All rights reserved.
//
// DS Data Systems and KonaKart and their respective logos, are 
// trademarks of DS Data Systems UK Ltd. All rights reserved.
//
// The information in this document is free software; you can redistribute 
// it and/or modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//

package com.konakart.bl.modules.payment.cod;

import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;

import com.konakart.app.KKConfiguration;
import com.konakart.app.KKException;
import com.konakart.app.Order;
import com.konakart.app.PaymentDetails;
import com.konakart.appif.KKEngIf;
import com.konakart.bl.modules.BaseModule;
import com.konakart.bl.modules.payment.BasePaymentModule;
import com.konakart.bl.modules.payment.PaymentInfo;
import com.konakart.bl.modules.payment.PaymentInterface;

/**
 * Cash On Delivery Module.
 * 
 */
public class Cod extends BasePaymentModule implements PaymentInterface
{
    private static int sortOrder = -1;

    private static int zone;

    private static int orderStatusId;

    private static String code = "cod";

    private static String bundleName = BaseModule.basePackage + ".payment.cod.Cod";

    private static HashMap<Locale,ResourceBundle> resourceBundleMap = new HashMap<Locale,ResourceBundle>();

    private static String mutex = "codMutex";

    // Configuration Keys
    private final static String MODULE_PAYMENT_COD_STATUS = "MODULE_PAYMENT_COD_STATUS";

    private final static String MODULE_PAYMENT_COD_ZONE = "MODULE_PAYMENT_COD_ZONE";

    private final static String MODULE_PAYMENT_COD_ORDER_STATUS_ID = "MODULE_PAYMENT_COD_ORDER_STATUS_ID";

    private final static String MODULE_PAYMENT_COD_SORT_ORDER = "MODULE_PAYMENT_COD_SORT_ORDER";

    // Message Catalogue Keys

    private final static String MODULE_PAYMENT_COD_TEXT_TITLE = "module.payment.cod.text.title";

    private final static String MODULE_PAYMENT_COD_TEXT_DESCRIPTION = "module.payment.cod.text.description";

    /**
     * Constructor
     * @param eng 
     * 
     * @throws KKException
     */
    public Cod(KKEngIf eng) throws KKException
    {
        super.init(eng);

        // Initialise the static variables
        if (sortOrder == -1)
        {
            synchronized (mutex)
            {
                if (sortOrder == -1)
                {
                    setStaticVariables();
                }
            }
        }
    }

    /**
     * Sets some static variables during setup
     * 
     * @throws KKException
     * 
     */
    public void setStaticVariables() throws KKException
    {
        KKConfiguration conf;

        conf = getEng().getConfiguration(MODULE_PAYMENT_COD_ZONE);
        if (conf == null)
        {
            zone = 0;
        } else
        {
            zone = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_COD_SORT_ORDER);
        if (conf == null)
        {
            sortOrder = 0;
        } else
        {
            sortOrder = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_COD_ORDER_STATUS_ID);
        if (conf == null)
        {
            orderStatusId = 0;
        } else
        {
            orderStatusId = new Integer(conf.getValue()).intValue();
        }
    }

    /**
     * Return a payment details object for Cash On Delivery
     * 
     * @param order
     * @param info
     * @return Returns information in a PaymentDetails object
     * @throws Exception
     */
    public PaymentDetails getPaymentDetails(Order order, PaymentInfo info) throws Exception
    {

        /*
         * The payPalZone zone, if greater than zero, should reference a GeoZone. If the
         * DeliveryAddress of the order isn't within that GeoZone, then we throw an exception
         */
        if (zone > 0)
        {
            checkZone(info, zone);
        }

        // Get the resource bundle
        ResourceBundle rb = getResourceBundle(mutex, bundleName, resourceBundleMap, info
                .getLocale());
        if (rb == null)
        {
            throw new KKException("A resource file cannot be found for the country "
                    + info.getLocale().getCountry());
        }

        PaymentDetails pDetails = new PaymentDetails();
        pDetails.setCode(code);
        pDetails.setSortOrder(sortOrder);
        pDetails.setOrderStatusId(orderStatusId);
        pDetails.setPaymentType(PaymentDetails.COD);
        pDetails.setDescription(rb.getString(MODULE_PAYMENT_COD_TEXT_DESCRIPTION));
        pDetails.setTitle(rb.getString(MODULE_PAYMENT_COD_TEXT_TITLE));

        return pDetails;
    }

    /**
     * Returns true or false
     * 
     * @throws KKException
     */
    public boolean isAvailable() throws KKException
    {
        return isAvailable(getEng(), MODULE_PAYMENT_COD_STATUS);
    }

}
